@@ -33,7 +33,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include "totem-remote.h"
-#include "scrsaver.h"
+#include "totem-scrsaver.h"
 #include "egg-recent-model.h"
 #include "egg-recent-view-gtk.h"
 #include "totem-playlist.h"
@@ -41,6 +41,9 @@
 #include "bacon-video-widget.h"
 #include "totem-skipto.h"
 
+#define totem_signal_block_by_data(obj, data) (g_signal_handlers_block_matched (obj, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, data))
+#define totem_signal_unblock_by_data(obj, data) (g_signal_handlers_unblock_matched (obj, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, data))
+
 typedef enum {
 	TOTEM_CONTROLS_VISIBLE,
 	TOTEM_CONTROLS_HIDDEN,
@@ -89,6 +92,8 @@ struct Totem {
 	/* Subtitles/Languages menus */
 	GtkWidget *subtitles;
 	GtkWidget *languages;
+	GList *subtitles_list;
+	GList *language_list;
 
 	/* exit fullscreen Popup */
 	GtkWidget *exit_popup;
@@ -114,8 +119,9 @@ struct Totem {
 	EggRecentModel *recent_model;
 	EggRecentViewGtk *recent_view;
 
-	/* Monitor to automatically unmount */
+	/* Monitor for playlist unmounts and drives/volumes monitoring */
 	GnomeVFSVolumeMonitor *monitor;
+	gboolean drives_changed;
 
 	/* session */
 	const char *argv0;
