@@ -15,6 +15,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ *
+ * The Totem project hereby grant permission for non-gpl compatible GStreamer
+ * plugins to be used and distributed together with GStreamer and Totem. This
+ * permission are above and beyond the permissions granted by the GPL license
+ * Totem is covered by.
+ *
+ * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * See license_change file for details.
+ *
  */
 
 #include <glib.h>
@@ -38,6 +47,7 @@ typedef enum {
 	TOTEM_REMOTE_COMMAND_UNKNOWN,
 	TOTEM_REMOTE_COMMAND_PLAY,
 	TOTEM_REMOTE_COMMAND_PAUSE,
+	TOTEM_REMOTE_COMMAND_PLAYPAUSE,
 	TOTEM_REMOTE_COMMAND_NEXT,
 	TOTEM_REMOTE_COMMAND_PREVIOUS,
 	TOTEM_REMOTE_COMMAND_SEEK_FORWARD,
