@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
- * Copyright (C) 2001-2004 Bastien Nocera <hadess@hadess.net>
+ * Copyright (C) 2001-2005 Bastien Nocera <hadess@hadess.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,14 +52,12 @@
 #include "totem-time-label.h"
 #include "totem-session.h"
 #include "totem-screenshot.h"
-#include "totem-sublang.h"
+#include "totem-menu.h"
 #include "totem-options.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
 #include "video-utils.h"
 
-#include "egg-recent-view.h"
-
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
@@ -84,9 +82,6 @@
 
 #define ZOOM_IN_OFFSET 1
 #define ZOOM_OUT_OFFSET -1
-#define ZOOM_UPPER 200
-#define ZOOM_RESET 100
-#define ZOOM_LOWER 10
 
 #define FULLSCREEN_POPUP_TIMEOUT 5 * 1000
 
@@ -115,7 +110,6 @@ static void show_controls (Totem *totem, gboolean was_fullscreen);
 static gboolean totem_is_fullscreen (Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
 static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
-static void popup_timeout_remove (Totem *totem);
 
 static void
 long_action (void)
@@ -168,8 +162,6 @@ totem_action_exit (Totem *totem)
 	if (totem == NULL)
 		exit (0);
 
-	popup_timeout_remove (totem);
-
 #ifndef HAVE_GTK_ONLY
 	totem_gromit_clear (TRUE);
 #endif /* !HAVE_GTK_ONLY */
@@ -184,6 +176,7 @@ totem_action_exit (Totem *totem)
 	if (totem->win)
 		gtk_widget_hide (totem->win);
 
+	totem_sublang_exit (totem);
 	totem_named_icons_dispose (totem);
 
 	if (totem->bvw)
@@ -280,6 +273,7 @@ totem_action_eject (Totem *totem)
 	GError *err = NULL;
 	char *cmd, *prefix;
 	const char *needle;
+	char *device;
 
 	needle = strchr (totem->mrl, ':');
 	g_assert (needle != NULL);
@@ -288,8 +282,11 @@ totem_action_eject (Totem *totem)
 	totem_playlist_clear_with_prefix (totem->playlist, prefix);
 	g_free (prefix);
 
-	cmd = g_strdup_printf ("eject %s", gconf_client_get_string
-			(totem->gc, GCONF_PREFIX"/mediadev", NULL));
+	g_object_get (G_OBJECT (totem->bvw),
+			"mediadev", &device, NULL);
+	cmd = g_strdup_printf ("eject %s", device);
+	g_free (device);
+
 	if (g_spawn_command_line_sync (cmd, NULL, NULL, NULL, &err) == FALSE)
 	{
 		totem_action_error (_("Totem could not eject the optical media."), err->message, totem);
@@ -307,6 +304,9 @@ totem_action_play (Totem *totem)
 	if (totem->mrl == NULL)
 		return;
 
+	if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+		return;
+
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
 	if (totem_is_fullscreen (totem) != FALSE)
@@ -355,8 +355,8 @@ totem_action_seek (Totem *totem, double pos)
 	}
 }
 
-static void
-totem_action_set_mrl_and_play (Totem *totem, char *mrl)
+void
+totem_action_set_mrl_and_play (Totem *totem, const char *mrl)
 {
 	if (totem_action_set_mrl (totem, mrl) != FALSE)
 		totem_action_play (totem);
@@ -442,34 +442,33 @@ totem_action_play_pause (Totem *totem)
 	}
 }
 
-static gboolean
-window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
-		Totem *totem)
+void
+totem_action_pause (Totem *totem)
 {
-	if (event->changed_mask != GDK_WINDOW_STATE_FULLSCREEN) {
-		return FALSE;
+	if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+	{
+		bacon_video_widget_pause (totem->bvw);
+		play_pause_set_label (totem, STATE_PAUSED);
+		totem_scrsaver_enable (totem->scr);
 	}
+}
 
-	if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
-		totem_action_save_size (totem);
-		update_fullscreen_size (totem);
-		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
-		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
-
-		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
-			totem_scrsaver_disable (totem->scr);
-
-		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
-		show_controls (totem, FALSE);
-	} else {
+void
+totem_action_fullscreen_toggle (Totem *totem)
+{
+	if (totem_is_fullscreen (totem) != FALSE)
+	{
 		GtkWidget *item;
 
 		popup_hide (totem);
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
+		gtk_window_unfullscreen (GTK_WINDOW(totem->win));
 		bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
 		totem_scrsaver_enable (totem->scr);
 
+		totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
+
 		item = glade_xml_get_widget
 			(totem->xml, "tmw_show_controls_menu_item");
 
@@ -482,18 +481,17 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
 			show_controls (totem, TRUE);
 		}
-	}
-
-	return FALSE;
-}
-
-void
-totem_action_fullscreen_toggle (Totem *totem)
-{
-	if (totem_is_fullscreen (totem) != FALSE) {
-		gtk_window_unfullscreen (GTK_WINDOW (totem->win));
 	} else {
-		gtk_window_fullscreen (GTK_WINDOW (totem->win));
+		totem_action_save_size (totem);
+		update_fullscreen_size (totem);
+		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
+		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
+		gtk_window_fullscreen (GTK_WINDOW(totem->win));
+		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+			totem_scrsaver_disable (totem->scr);
+
+		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
+		show_controls (totem, FALSE);
 	}
 }
 
@@ -589,7 +587,7 @@ update_mrl_label (Totem *totem, const char *name)
 		/* Update the mrl label */
 		text = g_strdup_printf
 			("<span size=\"medium\"><b>%s</b></span>",
-			 _("No file"));
+			 _("No File"));
 		widget = glade_xml_get_widget (totem->xml, "tcw_title_label");
 		gtk_label_set_markup (GTK_LABEL (widget), text);
 		widget = glade_xml_get_widget (totem->xml, "tmw_title_label");
@@ -847,60 +845,19 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 	}
 }
 
-static void
-totem_action_zoom (Totem *totem, int zoom)
-{
-	GtkWidget *item;
-
-	if (zoom < ZOOM_LOWER || zoom > ZOOM_UPPER)
-		return;
-
-	bacon_video_widget_set_zoom (totem->bvw, zoom);
-
-	if (zoom == ZOOM_UPPER) {
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_in_menu_item");
-		gtk_widget_set_sensitive (item, FALSE);
-	} else if (zoom == ZOOM_LOWER) {
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_out_menu_item");
-		gtk_widget_set_sensitive (item, FALSE);
-	} else if (zoom == ZOOM_RESET) {
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_reset_menu_item");
-		gtk_widget_set_sensitive (item, FALSE);
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_in_menu_item");
-		gtk_widget_set_sensitive (item, TRUE);
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_out_menu_item");
-		gtk_widget_set_sensitive (item, TRUE);
-	} else {
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_in_menu_item");
-		gtk_widget_set_sensitive (item, TRUE);
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_out_menu_item");
-		gtk_widget_set_sensitive (item, TRUE);
-		item = glade_xml_get_widget
-			(totem->xml, "tmw_zoom_reset_menu_item");
-		gtk_widget_set_sensitive (item, TRUE);
-	}
-}
-
 static void
 totem_action_zoom_relative (Totem *totem, int off_pct)
 {
 	int zoom;
 
 	zoom = bacon_video_widget_get_zoom (totem->bvw);
-	totem_action_zoom (totem, zoom + off_pct);
+	bacon_video_widget_set_zoom (totem->bvw, zoom + off_pct);
 }
 
 static void
 totem_action_zoom_reset (Totem *totem)
 {
-	totem_action_zoom (totem, 100);
+	bacon_video_widget_set_zoom (totem->bvw, 100);
 }
 
 void
@@ -1138,37 +1095,6 @@ on_playlist_button_toggled (GtkToggleButton *button, Totem *totem)
 		gtk_widget_hide (GTK_WIDGET (totem->playlist));
 }
 
-static void
-on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
-                         Totem *totem)
-{
-	char *uri;
-	gboolean playlist_changed;
-	guint end;
-
-	uri = egg_recent_item_get_uri (item);
-
-	totem_signal_block_by_data (totem->playlist, totem);
-
-	end = totem_playlist_get_last (totem->playlist);
-	playlist_changed = totem_playlist_add_mrl (totem->playlist, uri, NULL);
-	egg_recent_model_add_full (totem->recent_model, item);
-
-	totem_signal_unblock_by_data (totem->playlist, totem);
-
-	if (playlist_changed)
-	{
-		char *mrl;
-
-		totem_playlist_set_current (totem->playlist, end + 1);
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
-		g_free (mrl);
-	}
-
-	g_free (uri);
-}
-
 static void
 on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
@@ -1231,9 +1157,9 @@ on_error_event (BaconVideoWidget *bvw, char *message,
 		play_pause_set_label (totem, STATE_STOPPED);
 
 	if (fatal == FALSE) {
-		totem_action_error (_("An error occurred"), message, totem);
+		totem_action_error (_("An error occured"), message, totem);
 	} else {
-		totem_action_error_and_exit (_("An error occurred"),
+		totem_action_error_and_exit (_("An error occured"),
 				message, totem);
 	}
 }
@@ -1445,19 +1371,6 @@ vol_cb (GtkWidget *widget, Totem *totem)
 	}
 }
 
-static void
-totem_action_add_recent (Totem *totem, const char *filename)
-{
-	EggRecentItem *item;
-
-	if (strstr (filename, "file:///") == NULL)
-		return;
-
-	item = egg_recent_item_new_from_uri (filename);
-	egg_recent_item_add_group (item, "Totem");
-	egg_recent_model_add_full (totem->recent_model, item);
-}
-
 static void
 totem_add_cd_track_name (Totem *totem, const char *filename)
 {
@@ -1567,7 +1480,7 @@ on_open1_activate (GtkButton *button, Totem *totem)
 	int response;
 	static char *path = NULL;
 
-	fs = gtk_file_chooser_dialog_new (_("Select files"),
+	fs = gtk_file_chooser_dialog_new (_("Select Files"),
 			GTK_WINDOW (totem->win), GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
@@ -1673,37 +1586,6 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 	g_object_unref (glade);
 }
 
-static void
-on_play_disc1_activate (GtkButton *button, Totem *totem)
-{
-	MediaType type;
-	GError *error = NULL;
-	const gchar *device;
-
-	device = gconf_client_get_string (totem->gc,
-					  GCONF_PREFIX"/mediadev", NULL);
-	type = totem_cd_detect_type (device, &error);
-	switch (type) {
-		case MEDIA_TYPE_ERROR:
-			totem_action_error ("Failed to play Audio/Video Disc",
-					    error ? error->message : "Reason unknown",
-					    totem);
-			return;
-		case MEDIA_TYPE_DATA:
-			/* Maybe set default location to the mountpoint of
-			 * this device?... */
-			on_open1_activate (button, totem);
-			return;
-		case MEDIA_TYPE_DVD:
-		case MEDIA_TYPE_VCD:
-		case MEDIA_TYPE_CDDA:
-			totem_action_play_media (totem, type);
-			break;
-		default:
-			g_assert_not_reached ();
-	}
-}
-
 static void
 on_eject1_activate (GtkButton *button, Totem *totem)
 {
@@ -1836,9 +1718,6 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	GtkWidget *menubar, *controlbar, *statusbar, *item, *bvw_vbox, *widget;
 	int width = 0, height = 0;
 
-	if (totem->bvw == NULL)
-		return;
-
 	menubar = glade_xml_get_widget (totem->xml, "tmw_menubar");
 	controlbar = glade_xml_get_widget (totem->xml, "tmw_controls_vbox");
 	statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
@@ -2027,7 +1906,7 @@ on_take_screenshot1_activate (GtkButton *button, Totem *totem)
 	pixbuf = bacon_video_widget_get_current_frame (totem->bvw);
 	if (pixbuf == NULL)
 	{
-		totem_action_error (_("Totem could not get a screenshot of that film."), _("Please file a bug, this isn't supposed to happen."), totem);
+		totem_action_error (_("Totem could not get a screenshot of that film."), _("This is not supposed to happen; please file a bug report."), totem);
 		return;
 	}
 
@@ -2225,9 +2104,12 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_PLAY:
 		totem_action_play (totem);
 		break;
-	case TOTEM_REMOTE_COMMAND_PAUSE:
+	case TOTEM_REMOTE_COMMAND_PLAYPAUSE:
 		totem_action_play_pause (totem);
 		break;
+	case TOTEM_REMOTE_COMMAND_PAUSE:
+		totem_action_pause (totem);
+		break;
 	case TOTEM_REMOTE_COMMAND_SEEK_FORWARD:
 		totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET);
 		break;
@@ -2297,10 +2179,8 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		break;
 	}
 
-	if (handled != FALSE &&
-			gtk_window_is_active (GTK_WINDOW (totem->win))) {
+	if (handled != FALSE)
 		on_video_motion_notify_event (NULL, NULL, totem);
-	}
 }
 
 #ifdef HAVE_REMOTE
@@ -2403,12 +2283,10 @@ playlist_shuffle_toggle_cb (TotemPlaylist *playlist, gboolean shuffle, Totem *to
 static void
 update_fullscreen_size (Totem *totem)
 {
-	GdkScreen *screen;
-
-	screen = gtk_window_get_screen (GTK_WINDOW (totem->win));
-	gdk_screen_get_monitor_geometry (screen,
+	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
 			gdk_screen_get_monitor_at_window
-			(screen, totem->win->window),
+			(gdk_screen_get_default (),
+			 totem->win->window),
 			&totem->fullscreen_rect);
 }
 
@@ -2424,8 +2302,6 @@ move_popups (Totem *totem)
 	int control_width, control_height;
 	int exit_width, exit_height;
 
-	update_fullscreen_size (totem);
-
 	gtk_window_get_size (GTK_WINDOW (totem->control_popup),
 			&control_width, &control_height);
 	gtk_window_get_size (GTK_WINDOW (totem->exit_popup),
@@ -2443,20 +2319,23 @@ move_popups (Totem *totem)
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.width - control_width,
-				totem->fullscreen_rect.height + totem->fullscreen_rect.y - control_height);
+				totem->fullscreen_rect.height
+				- control_height);
 	} else {
 		gtk_window_move (GTK_WINDOW (totem->exit_popup),
-				totem->fullscreen_rect.width + totem->fullscreen_rect.x - exit_width,
+				totem->fullscreen_rect.width - exit_width,
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.x,
-				totem->fullscreen_rect.height + totem->fullscreen_rect.y - control_height);
+				totem->fullscreen_rect.height
+				- control_height);
 	}
 }
 
 static void
 size_changed_cb (GdkScreen *screen, Totem *totem)
 {
+	update_fullscreen_size (totem);
 	move_popups (totem);
 }
 
@@ -2612,6 +2491,7 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 		totem_gromit_clear (FALSE);
 		break;
 #endif /* !HAVE_GTK_ONLY */
+	case GDK_F11:
 	case GDK_f:
 	case GDK_F:
 		totem_action_fullscreen_toggle (totem);
@@ -2934,9 +2814,6 @@ totem_callback_connect (Totem *totem)
 			"tmw_open_location_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_open_location1_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_play_disc_menu_item");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_play_disc1_activate), totem);
 	item = glade_xml_get_widget (totem->xml, "tmw_eject_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_eject1_activate), totem);
@@ -3118,8 +2995,6 @@ totem_callback_connect (Totem *totem)
 	g_object_notify (G_OBJECT (totem->win), "is-active");
 	g_signal_connect_swapped (G_OBJECT (totem->win), "notify",
 			G_CALLBACK (popup_hide), totem);
-	g_signal_connect (G_OBJECT (totem->win), "window-state-event",
-			G_CALLBACK (window_state_event_cb), totem);
 
 	/* Screen size and Theme changes */
 	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
@@ -3418,39 +3293,6 @@ totem_time_display_create (void)
 	return widget;
 }
 
-static void
-totem_setup_recent (Totem *totem)
-{
-	GtkWidget *menu_item;
-	GtkWidget *menu;
-
-	menu_item = glade_xml_get_widget (totem->xml, "tmw_menu_item_movie");
-	menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
-	menu_item = glade_xml_get_widget (totem->xml,
-			"tmw_menu_recent_separator");
-
-	g_return_if_fail (menu != NULL);
-	g_return_if_fail (menu_item != NULL);
-
-	/* it would be better if we just filtered by mime-type, but there
-	 * doesn't seem to be an easy way to figure out which mime-types we
-	 * can handle */
-	totem->recent_model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
-
-	totem->recent_view = egg_recent_view_gtk_new (menu, menu_item);
-	egg_recent_view_gtk_show_icons (EGG_RECENT_VIEW_GTK
-			(totem->recent_view), FALSE);
-	egg_recent_model_set_limit (totem->recent_model, 5);
-	egg_recent_view_set_model (EGG_RECENT_VIEW (totem->recent_view),
-			totem->recent_model);
-	egg_recent_model_set_filter_groups (totem->recent_model,
-			"Totem", NULL);
-	egg_recent_view_gtk_set_trailing_sep (totem->recent_view, TRUE);
-
-	g_signal_connect (totem->recent_view, "activate",
-			G_CALLBACK (on_recent_file_activate), totem);
-}
-
 static void
 totem_message_connection_receive_cb (const char *msg, Totem *totem)
 {
@@ -3489,7 +3331,7 @@ main (int argc, char **argv)
 	{
 		gtk_init (&argc, &argv);
 		g_set_application_name (_("Totem Movie Player"));
-		totem_action_error_and_exit (_("Could not initialise the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
+		totem_action_error_and_exit (_("Could not initialize the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
 	}
 
 	g_thread_init (NULL);
@@ -3506,11 +3348,13 @@ main (int argc, char **argv)
 #endif /* HAVE_GTK_ONLY */
 
 	g_set_application_name (_("Totem Movie Player"));
+
 	gnome_vfs_init ();
 
-	if ((gc = gconf_client_get_default ()) == NULL)
+	gc = gconf_client_get_default ();
+	if (gc == NULL)
 	{
-		totem_action_error_and_exit (_("Totem couln't initialise the configuration engine."), _("Make sure that GNOME is properly installed."), NULL);
+		totem_action_error_and_exit (_("Totem could not initialize the configuration engine."), _("Make sure that GNOME is properly installed."), NULL);
 	}
 
 #ifndef HAVE_GTK_ONLY
@@ -3594,9 +3438,10 @@ main (int argc, char **argv)
 
 	totem_session_setup (totem, argv);
 	totem_setup_recent (totem);
+	totem_setup_file_monitoring (totem);
+	totem_setup_play_disc (totem);
 	totem_callback_connect (totem);
 	totem_setup_window (totem);
-	totem_setup_file_monitoring (totem);
 
 	/* Show ! gtk_main_iteration trickery to show all the widgets
 	 * we have so far */
